/**
 * Benchmark dependencies
 */
var Benchmark = require('benchmark')
  , microtime = require('microtime')
  , http = require('http')
  , path = require('path')
  , fs = require('fs')
  , common = require('../tests/common');
  
/**
 * Different memcached drivers
 */
var Memcached = require('../');

/**
 * Generate data that will be used for testing
 */
var tinyString = common.alphabet(12)
  , smallString = common.alphabet(1E3)
  , mediumString = common.alphabet(25E3)
  , largeString = fs.readFileSync(path.join(__dirname, '../tests/fixtures/lipsum.txt'));

/**
 * Setup the different benchmarks and stress tests
 */
var suites = {}
  , memcached = new Memcached(common.servers.single);
  
/**
 * Benchmark setting of tiny strings
 */
suites.tinySet = new Benchmark.Suite('set tiny strings');
suites.tinySet
// memcached client
.add('Memcached', function(){
  // fire and forget
  memcached.set('benchmark:set:1', tinyString, 0, function(){});
})

// output logging
.on('cycle', function(bench){
  console.log("Executing benchmark:" + bench);
})
.on('complete', function(){
  console.log('Fastest Memcached driver for setting a `tiny` string is ' + this.filter('fastest').pluck('name')+ '\n');
  
  setTimeout(function(){
    suites.smallSet.run();
  }, 5000); // let the memcache server rest for a while, before we hit it again
});

/**
 * Benchmark setting of small strings
 */
suites.smallSet = new Benchmark.Suite('set small strings');
suites.smallSet

// memcached client
.add('Memcached', function(){
  // fire and forget
  memcached.set('benchmark:set:2', smallString, 0, function(){});
})

// output logging
.on('cycle', function(bench){
  console.log("Executing benchmark:" + bench);
})
.on('complete', function(){
  
  console.log('Fastest Memcached driver for setting a `small` string is ' + this.filter('fastest').pluck('name')+ '\n');
  
  setTimeout(function(){
    suites.mediumSet.run();
  }, 5000); // let the memcache server rest for a while, before we hit it again
});

/**
 * Benchmark setting of medium strings
 */
suites.mediumSet = new Benchmark.Suite('set medium strings');
suites.mediumSet

// memcached client
.add('Memcached', function(){
  // fire and forget
  memcached.set('benchmark:set:3', mediumString, 0, function(){});
})

// output logging
.on('cycle', function(bench){
  console.log("Executing benchmark:" + bench);
})
.on('complete', function(){
  console.log('Fastest Memcached driver for setting a `medium` string is ' + this.filter('fastest').pluck('name')+ '\n');
    
  setTimeout(function(){
    suites.largeSet.run();
  }, 5000); // let the memcache server rest for a while, before we hit it again
});

/**
 * Benchmark setting of medium strings
 */
suites.largeSet = new Benchmark.Suite('set large strings');
suites.largeSet

// memcached client
.add('Memcached', function(){
  // fire and forget
  memcached.set('benchmark:set:3', largeString, 0, function(){});
})

// output logging
.on('cycle', function(bench){
  console.log("Executing benchmark:" + bench);
})
.on('complete', function(){
  console.log('Fastest Memcached driver for setting a `large` string is ' + this.filter('fastest').pluck('name')+ '\n');
  
  setTimeout(function(){
    suites.tinyGet.run();
  }, 5000); // let the memcache server rest for a while, before we hit it again
});

/**
 * Benchmark setting of tiny strings
 */
suites.tinyGet = new Benchmark.Suite('get tiny strings');
suites.tinyGet
// memcached client
.add('Memcached', function(){
  // fire and forget
  memcached.get('benchmark:set:1', function(){});
})

// output logging
.on('cycle', function(bench){
  console.log("Executing benchmark:" + bench);
})
.on('complete', function(){
  console.log('Fastest Memcached driver for getting a `tiny` string is ' + this.filter('fastest').pluck('name')+ '\n');
  
  setTimeout(function(){
    suites.smallGet.run();
  }, 5000); // let the memcache server rest for a while, before we hit it again
});

/**
 * Benchmark setting of small strings
 */
suites.smallGet = new Benchmark.Suite('get small strings');
suites.smallGet

// memcached client
.add('Memcached', function(){
  // fire and forget
  memcached.get('benchmark:set:2', function(){});
})

// output logging
.on('cycle', function(bench){
  console.log("Executing benchmark:" + bench);
})
.on('complete', function(){
  
  console.log('Fastest Memcached driver for getting a `small` string is ' + this.filter('fastest').pluck('name')+ '\n');
  
  setTimeout(function(){
    suites.mediumGet.run();
  }, 5000); // let the memcache server rest for a while, before we hit it again
});

/**
 * Benchmark setting of medium strings
 */
suites.mediumGet = new Benchmark.Suite('get medium strings');
suites.mediumGet

// memcached client
.add('Memcached', function(){
  // fire and forget
  memcached.get('benchmark:get:3', function(){});
})

// output logging
.on('cycle', function(bench){
  console.log("Executing benchmark: " + bench);
})
.on('complete', function(){
  console.log('Fastest Memcached driver for getting a `medium` string is ' + this.filter('fastest').pluck('name') + '\n');
    
  setTimeout(function(){
    suites.largeGet.run();
  }, 5000); // let the memcache server rest for a while, before we hit it again
});

/**
 * Benchmark setting of medium strings
 * /
suites.largeGet = new Benchmark.Suite('get large strings');
suites.largeGet

// memcached client
.add('Memcached', function(){
  // fire and forget
  memcached.get('benchmark:set:3', function(){});
})

// output logging
.on('cycle', function(bench){
  console.log("Executing benchmark:" + bench);
})
.on('complete', function(){
  console.log('Fastest Memcached driver for getting a `large` string is ' + this.filter('fastest').pluck('name')+ '\n');
  process.exit();
});*/

/**
 * Run the suites
 */
suites.tinySet.run();